@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
@@ -65,6 +69,87 @@
 /* Definitions */
 #define LOG_FF_DAT _olsrv2_ffdat_subsystem.logging
 
+/**
+ * Configuration settings of DATFF Metric
+ */
+struct ff_dat_config {
+  /*! Interval between two updates of the metric */
+  uint64_t interval;
+
+  /*! length of history in 'interval sized' memory cells */
+  int32_t window;
+
+  /*! true if metric should include link speed */
+  bool ett;
+
+  /*! selects how loss should be scaled */
+  int loss_exponent;
+
+  /*! true if MIC factor should be applied to metric */
+  bool mic;
+
+#ifdef COLLECT_RAW_DATA
+  /* filename to store raw data into */
+  char *rawdata_file;
+
+  /* true if metric should collect raw data */
+  bool rawdata_start;
+
+  /* time in milliseconds until measurement stops */
+  uint64_t rawdata_maxtime;
+
+  /* maxmimum number of measured packets until measurement stops */
+  int rawdata_maxpackets;
+#endif
+};
+
+/**
+ * a single history memory cell, stores the metric
+ * data for a single update interval
+ */
+struct link_datff_bucket {
+  /*! number of RFC5444 packets received in time interval */
+  int received;
+
+  /*! sum of received and lost RFC5444 packets in time interval */
+  int total;
+
+  /*! link speed scaled to "minimum speed = 1" */
+  int scaled_speed;
+};
+
+/**
+ * Additional data for a nhdp_link class for metric calculation
+ */
+struct link_datff_data {
+  /*! timer for measuring lost hellos when no further packets are received */
+  struct oonf_timer_instance hello_lost_timer;
+
+  /*! back pointer to NHDP link */
+  struct nhdp_link *nhdp_link;
+
+  /*! current position in history ringbuffer */
+  int activePtr;
+
+  /*! number of missed hellos based on timeouts since last received packet */
+  int missed_hellos;
+
+  /*! last received packet sequence number */
+  uint16_t last_seq_nr;
+
+  /*! remember the last transmitted packet loss for hysteresis (scaled by 1000) */
+  int64_t last_packet_success_rate;
+
+  /*! last known hello interval */
+  uint64_t hello_interval;
+
+  /*! estimated number of neighbors of this link */
+  uint32_t link_neigborhood;
+
+  /*! history ringbuffer */
+  struct link_datff_bucket buckets[0];
+};
+
 /* prototypes */
 static int _init(void);
 static void _cleanup(void);
@@ -76,15 +161,15 @@ static void _cb_link_added(void *);
 static void _cb_link_changed(void *);
 static void _cb_link_removed(void *);
 
-static void _cb_dat_sampling(void *);
-void _calculate_link_neighborhood(struct nhdp_link *lnk,
+static void _cb_dat_sampling(struct oonf_timer_instance *);
+static void _calculate_link_neighborhood(struct nhdp_link *lnk,
     struct link_datff_data *ldata);
-int _calculate_loss_exponent(int link_neigborhood);
+static int _calculate_dynamic_loss_exponent(int link_neigborhood);
 static uint32_t _apply_packet_loss(struct nhdp_link *lnk,
     struct link_datff_data *ldata,
     uint32_t metric, uint32_t received, uint32_t total);
 
-static void _cb_hello_lost(void *);
+static void _cb_hello_lost(struct oonf_timer_instance *);
 
 static enum rfc5444_result _cb_process_packet(
       struct rfc5444_reader_tlvblock_context *context);
@@ -103,13 +188,24 @@ static int _cb_cfg_validate(const char *section_name,
 static void _cb_cfg_changed(void);
 
 /* plugin declaration */
+
+/**
+ * loss scaling options
+ */
 enum idx_loss_scaling {
+  /*! linear loss scaling */
   IDX_LOSS_LINEAR,
+
+  /*! quadratic loss scaling */
   IDX_LOSS_QUADRATIC,
+
+  /*! cubic loss scaling */
   IDX_LOSS_CUBIC,
+
+  /*! dynamic loss scaling */
   IDX_LOSS_DYNAMIC,
 };
-const char *LOSS_SCALING[] = {
+static const char *LOSS_SCALING[] = {
   [IDX_LOSS_LINEAR]    = "linear",
   [IDX_LOSS_QUADRATIC] = "quadratic",
   [IDX_LOSS_CUBIC]     = "cubic",
@@ -216,8 +312,6 @@ static struct nhdp_domain_metric _datff_handler = {
   .metric_minimum = DATFF_LINKCOST_MINIMUM,
   .metric_maximum = DATFF_LINKCOST_MAXIMUM,
 
-  .incoming_link_start = DATFF_LINKCOST_START,
-
   .link_to_string = _link_to_string,
   .path_to_string = _path_to_string,
   .internal_link_to_string = _int_link_to_string,
@@ -250,7 +344,7 @@ _init(void) {
   oonf_timer_add(&_sampling_timer_info);
   oonf_timer_add(&_hello_lost_info);
 
-  _protocol = oonf_rfc5444_add_protocol(RFC5444_PROTOCOL, true);
+  _protocol = oonf_rfc5444_get_default_protocol();
 
   oonf_rfc5444_add_protocol_pktseqno(_protocol);
 #ifdef COLLECT_RAW_DATA
@@ -279,7 +373,7 @@ _cleanup(void) {
   nhdp_domain_metric_remove(&_datff_handler);
 
   oonf_rfc5444_remove_protocol_pktseqno(_protocol);
-  oonf_rfc5444_remove_protocol(_protocol);
+  _protocol = NULL;
 
   oonf_class_extension_remove(&_link_extenstion);
 
@@ -335,7 +429,9 @@ _cb_link_added(void *ptr) {
 
   /* start 'hello lost' timer for link */
   data->hello_lost_timer.class = &_hello_lost_info;
-  data->hello_lost_timer.cb_context = ptr;
+
+  /* minimal value possible for success rate */
+  data->last_packet_success_rate = 1000ll;
 }
 
 /**
@@ -419,8 +515,7 @@ _get_median_rx_linkspeed(struct link_datff_data *ldata) {
  */
 static int
 _get_scaled_rx_linkspeed(struct nhdp_link *lnk) {
-  // const struct oonf_linkconfig_data *linkdata;
-  struct os_interface_data *ifdata;
+  struct os_interface *os_if;
   const struct oonf_layer2_data *l2data;
   int rate;
 
@@ -430,13 +525,10 @@ _get_scaled_rx_linkspeed(struct nhdp_link *lnk) {
   }
 
   /* get local interface data  */
-  ifdata = oonf_interface_get_data(nhdp_interface_get_name(lnk->local_if), NULL);
-  if (!ifdata) {
-    return 1;
-  }
+  os_if = nhdp_interface_get_if_listener(lnk->local_if)->data;
 
   l2data = oonf_layer2_neigh_query(
-      ifdata->name, &lnk->remote_mac, OONF_LAYER2_NEIGH_RX_BITRATE);
+      os_if->name, &lnk->remote_mac, OONF_LAYER2_NEIGH_RX_BITRATE);
   if (!l2data) {
     return 1;
   }
@@ -460,7 +552,7 @@ _reset_missed_hello_timer(struct link_datff_data *);
  * @param ptr nhdp link
  */
 static void
-_cb_dat_sampling(void *ptr __attribute__((unused))) {
+_cb_dat_sampling(struct oonf_timer_instance *ptr __attribute__((unused))) {
   struct rfc7181_metric_field encoded_metric;
   struct link_datff_data *ldata;
   struct nhdp_link *lnk;
@@ -510,11 +602,6 @@ _cb_dat_sampling(void *ptr __attribute__((unused))) {
       }
     }
 
-    if (total == 0 || received == 0) {
-      nhdp_domain_set_incoming_metric(&_datff_handler, lnk, RFC7181_METRIC_MAX);
-      continue;
-    }
-
     /* update link speed */
     ldata->buckets[ldata->activePtr].scaled_speed = _get_scaled_rx_linkspeed(lnk);
 #ifdef COLLECT_RAW_DATA
@@ -544,7 +631,8 @@ _cb_dat_sampling(void *ptr __attribute__((unused))) {
     }
 
     /* calculate frame loss, use discrete values */
-    if (received * DATFF_FRAME_SUCCESS_RANGE <= total) {
+    if (total == 0 || received == 0
+        || received * DATFF_FRAME_SUCCESS_RANGE <= total) {
       metric *= DATFF_FRAME_SUCCESS_RANGE;
     }
     else {
@@ -593,7 +681,12 @@ _cb_dat_sampling(void *ptr __attribute__((unused))) {
   }
 }
 
-void
+/**
+ * Calculate how many neigbors a link has
+ * @param lnk nhdp link
+ * @param data ff data link data
+ */
+static void
 _calculate_link_neighborhood(struct nhdp_link *lnk, struct link_datff_data *data) {
   struct nhdp_l2hop *l2hop;
   struct nhdp_laddr *laddr;
@@ -613,8 +706,13 @@ _calculate_link_neighborhood(struct nhdp_link *lnk, struct link_datff_data *data
   data->link_neigborhood = count;
 }
 
-int
-_calculate_loss_exponent(int link_neigborhood) {
+/**
+ * Calculate the loss exponentiation based on the link neigborhood size
+ * @param link_neigborhood link neighborhood count
+ * @return loss exponent
+ */
+static int
+_calculate_dynamic_loss_exponent(int link_neigborhood) {
   if (link_neigborhood < 4) {
     return 1;
   }
@@ -641,20 +739,25 @@ _apply_packet_loss(struct nhdp_link *lnk,
     struct link_datff_data *ldata, uint32_t metric,
     uint32_t received, uint32_t total) {
   int64_t success_scaled_by_1000;
-  int64_t last_scaled_by_1000;
   int loss_exponent;
+  int64_t tmp_metric;
 
-  last_scaled_by_1000 = (int64_t)ldata->last_packet_success_rate * 1000ll;
-  success_scaled_by_1000 = ((int64_t)DATFF_FRAME_SUCCESS_RANGE * 1000ll) * received / total;
+  if (received * DATFF_FRAME_SUCCESS_RANGE < total) {
+    success_scaled_by_1000 = 1000ll;
+  }
+  else {
+    success_scaled_by_1000 = (((int64_t)DATFF_FRAME_SUCCESS_RANGE * 1000ll) * received) / total;
+  }
 
-  if (success_scaled_by_1000 >= last_scaled_by_1000 - 750
-      && success_scaled_by_1000 <= last_scaled_by_1000 + 750) {
+  if (success_scaled_by_1000 >= ldata->last_packet_success_rate - 750
+      && success_scaled_by_1000 <= ldata->last_packet_success_rate + 750
+      ) {
     /* keep old loss rate */
-    success_scaled_by_1000 = last_scaled_by_1000;
+    success_scaled_by_1000 = ldata->last_packet_success_rate;
   }
   else {
     /* remember new loss rate */
-    ldata->last_packet_success_rate = success_scaled_by_1000/1000;
+    ldata->last_packet_success_rate = success_scaled_by_1000;
   }
 
   _calculate_link_neighborhood(lnk, ldata);
@@ -670,35 +773,41 @@ _apply_packet_loss(struct nhdp_link *lnk,
       loss_exponent = 3;
       break;
     case IDX_LOSS_DYNAMIC:
-      loss_exponent = _calculate_loss_exponent(ldata->link_neigborhood);
+      loss_exponent = _calculate_dynamic_loss_exponent(ldata->link_neigborhood);
       break;
     default:
       loss_exponent = 1;
       break;
   }
 
+  tmp_metric = metric;
   while (loss_exponent) {
-    metric = ((int64_t)metric * (int64_t)DATFF_FRAME_SUCCESS_RANGE * 1000ll + 500ll) / success_scaled_by_1000;
+    tmp_metric = (tmp_metric * (int64_t)DATFF_FRAME_SUCCESS_RANGE * 1000ll + 500ll) / success_scaled_by_1000;
     loss_exponent--;
   }
 
-  if (_datff_config.mic) {
-    metric = metric * (int64_t)ldata->link_neigborhood;
+  if (_datff_config.mic && ldata->link_neigborhood > 1) {
+    tmp_metric = tmp_metric * (int64_t)ldata->link_neigborhood;
   }
-  return metric;
+
+  if (tmp_metric > RFC7181_METRIC_MAX) {
+    return RFC7181_METRIC_MAX;
+  }
+  if (tmp_metric < RFC7181_METRIC_MIN) {
+    return RFC7181_METRIC_MIN;
+  }
+  return tmp_metric;
 }
 
 /**
  * Callback triggered when the next hellos should have been received
- * @param ptr nhdp link
+ * @param ptr timer instance that fired
  */
 static void
-_cb_hello_lost(void *ptr) {
+_cb_hello_lost(struct oonf_timer_instance *ptr) {
   struct link_datff_data *ldata;
-  struct nhdp_link *lnk;
 
-  lnk = ptr;
-  ldata = oonf_class_get_extension(&_link_extenstion, lnk);
+  ldata = container_of(ptr, struct link_datff_data, hello_lost_timer);
 
   if (ldata->activePtr != -1) {
     ldata->missed_hellos++;
@@ -848,6 +957,10 @@ static const char *
 _path_to_string(struct nhdp_metric_str *buf, uint32_t metric, uint8_t hopcount) {
   struct nhdp_metric_str mbuf;
 
+  if (hopcount == 0) {
+    /* prevent division by zero */
+    hopcount = 1;
+  }
   snprintf(buf->buf, sizeof(*buf), "%s (%u hops)",
       _link_to_string(&mbuf, metric / hopcount), hopcount);
   return buf->buf;
@@ -867,7 +980,7 @@ _int_link_to_string(struct nhdp_metric_str *buf, struct nhdp_link *lnk) {
   }
 
   snprintf(buf->buf, sizeof(*buf), "p_recv=%"PRId64",p_total=%"PRId64","
-      "speed=%"PRId64",success=%u,missed_hello=%d,lastseq=%u,lneigh=%d",
+      "speed=%"PRId64",success=%"PRId64",missed_hello=%d,lastseq=%u,lneigh=%d",
       received, total, (int64_t)_get_median_rx_linkspeed(ldata) * (int64_t)1024,
       ldata->last_packet_success_rate, ldata->missed_hellos,
       ldata->last_seq_nr, ldata->link_neigborhood);
